/**
 * 此文件可能会根据现实情况做调整
 * update：2022.11.1
 * @type {{parserOptions: {ecmaVersion: number, parser: string, sourceType: string, ecmaFeatures: {jsx: boolean}}, parser: string, extends: string[], plugins: string[], root: boolean, globals: {defineEmits: string, defineProps: string, defineExpose: string, withDefaults: string}, rules: {"prettier/prettier": string, "space-before-blocks": string, "no-unreachable": string, "no-func-assign": string, "no-shadow": string, "no-self-compare": string, "no-return-assign": string, "@typescript-eslint/no-empty-function": string, "@typescript-eslint/no-var-requires": string, "@typescript-eslint/no-use-before-define": string, "no-mixed-spaces-and-tabs": string, "no-useless-escape": string, "arrow-spacing": string, "no-empty-function": string, "@typescript-eslint/no-non-null-assertion": string, "no-self-assign": string, "no-irregular-whitespace": string, "@typescript-eslint/explicit-function-return-type": string, camelcase: string, "switch-colon-spacing": string, "space-unary-ops": string, "space-infix-ops": string, "no-fallthrough": string, "no-delete-var": string, "newline-per-chained-call": [string,{ignoreChainWithDepth: number}], "no-useless-return": string, "max-statements-per-line": [string,{max: number}], "max-depth": string, "space-in-parens": string, "@typescript-eslint/no-unused-vars": string, eqeqeq: string, indent: string, "no-multi-spaces": string, "array-bracket-spacing": string, "default-case": string, "no-prototype-builtins": string, "@typescript-eslint/ban-types": string, "no-console": string, "no-lonely-if": string, "no-multiple-empty-lines": string, "max-nested-callbacks": (string|number)[], "@typescript-eslint/ban-ts-ignore": string, "@typescript-eslint/no-explicit-any": string, "no-extra-boolean-cast": string, "max-statements": (string|number)[], semi: string[], "prefer-rest-params": string, "no-else-return": string, "no-case-declarations": string, "no-async-promise-executor": string, "no-empty": string, curly: string, "no-debugger": string, "no-extra-parens": string, "no-lone-blocks": string, "max-params": (string|number)[], "no-redeclare": string, "no-return-await": string, "no-useless-catch": string, "@typescript-eslint/explicit-module-boundary-types": string, "dot-notation": string, "no-duplicate-case": string, "brace-style": string, "prefer-const": string, "@typescript-eslint/ban-ts-comment": string, "no-var": string}, env: {node: boolean, browser: boolean, es2021: boolean}}}
 */
module.exports = {
  root: true,
  env: {
    browser: true,
    node: true,
    es2021: true,
  },
  parser: "vue-eslint-parser",
  extends: [
    "eslint:recommended",
    "plugin:vue/vue3-recommended",
    "plugin:@typescript-eslint/recommended",
    "plugin:prettier/recommended",
    // eslint-config-prettier 的缩写
    "prettier",
  ],
  parserOptions: {
    ecmaVersion: 12,
    parser: "@typescript-eslint/parser",
    sourceType: "module",
    ecmaFeatures: {
      jsx: true,
    },
  },
  // eslint-plugin-vue @typescript-eslint/eslint-plugin eslint-plugin-prettier的缩写
  plugins: ["vue", "@typescript-eslint", "prettier"],
  rules: {
    "@typescript-eslint/ban-ts-ignore": "off",
    "@typescript-eslint/no-unused-vars": "off",
    "@typescript-eslint/explicit-function-return-type": "off",
    "@typescript-eslint/no-explicit-any": "off",
    "@typescript-eslint/no-var-requires": "off",
    "@typescript-eslint/no-empty-function": "off",
    "@typescript-eslint/no-use-before-define": "off",
    "@typescript-eslint/ban-ts-comment": "off",
    "@typescript-eslint/ban-types": "off",
    "@typescript-eslint/no-non-null-assertion": "off",
    "@typescript-eslint/explicit-module-boundary-types": "off",
    "prettier/prettier": "error",
    "no-var": "error",
    // 禁止出现console
    "no-console": "warn",
    // 禁用debugger
    "no-debugger": "warn",
    // 禁止出现重复的 case 标签
    "no-duplicate-case": "warn",
    // 禁止出现空语句块
    "no-empty": "warn",
    // 禁止不必要的括号
    "no-extra-parens": "off",
    // 禁止对 function 声明重新赋值
    "no-func-assign": "warn",
    // 禁止在 return、throw、continue 和 break 语句之后出现不可达代码
    "no-unreachable": "warn",
    // 强制所有控制语句使用一致的括号风格
    curly: "warn",
    // 要求 switch 语句中有 default 分支
    "default-case": "warn",
    // 强制尽可能地使用点号
    "dot-notation": "warn",
    // 要求使用 === 和 !==
    eqeqeq: "error",
    // 禁止 if 语句中 return 语句之后有 else 块
    "no-else-return": "warn",
    // 禁止出现空函数
    "no-empty-function": "warn",
    // 禁用不必要的嵌套块
    "no-lone-blocks": "warn",
    // 禁止使用多个空格
    "no-multi-spaces": "warn",
    // 禁止多次声明同一变量
    "no-redeclare": "warn",
    // 禁止在 return 语句中使用赋值语句
    "no-return-assign": "warn",
    // 禁用不必要的 return await
    "no-return-await": "warn",
    // 禁止自我赋值
    "no-self-assign": "warn",
    // 禁止自身比较
    "no-self-compare": "warn",
    // 禁止不必要的 catch 子句
    "no-useless-catch": "warn",
    // 禁止多余的 return 语句
    "no-useless-return": "warn",
    // 禁止变量声明与外层作用域的变量同名
    "no-shadow": "off",
    // 允许delete变量
    "no-delete-var": "off",
    // 强制数组方括号中使用一致的空格
    "array-bracket-spacing": "warn",
    // 强制在代码块中使用一致的大括号风格
    "brace-style": "warn",
    // 强制使用骆驼拼写法命名约定
    camelcase: "warn",
    // 强制使用一致的缩进
    indent: "off",
    // 强制在 JSX 属性中一致地使用双引号或单引号
    // 'jsx-quotes': 'warn',
    // 强制可嵌套的块的最大深度4
    "max-depth": "warn",
    // 强制最大行数 999
    "max-lines": ["error", { "max": 999 }],
    // 强制函数最大代码行数 100
    // 'max-lines-per-function': ['warn', { max: 100 }],
    // 强制函数块最多允许的的语句数量100
    "max-statements": ["warn", 100],
    // 强制回调函数最大嵌套深度
    "max-nested-callbacks": ["warn", 3],
    // 强制函数定义中最多允许的参数数量
    "max-params": ["warn", 3],
    // 强制每一行中所允许的最大语句数量
    "max-statements-per-line": ["warn", { max: 1 }],
    // 要求方法链中每个调用都有一个换行符
    "newline-per-chained-call": ["warn", { ignoreChainWithDepth: 3 }],
    // 禁止 if 作为唯一的语句出现在 else 语句中
    "no-lonely-if": "warn",
    // 禁止空格和 tab 的混合缩进
    "no-mixed-spaces-and-tabs": "warn",
    // 禁止出现多行空行
    "no-multiple-empty-lines": "warn",
    // 默认出现;
    semi: ["error", "always"],
    // 强制在块之前使用一致的空格
    "space-before-blocks": "warn",
    // 强制在 function的左括号之前使用一致的空格
    // 'space-before-function-paren': ['warn', 'never'],
    // 强制在圆括号内使用一致的空格
    "space-in-parens": "warn",
    // 要求操作符周围有空格
    "space-infix-ops": "warn",
    // 强制在一元操作符前后使用一致的空格
    "space-unary-ops": "warn",
    // 强制在注释中 // 或 /* 使用一致的空格
    // "spaced-comment": "warn",
    // 强制在 switch 的冒号左右有空格
    "switch-colon-spacing": "warn",
    // 强制箭头函数的箭头前后使用一致的空格
    "arrow-spacing": "warn",
    "prefer-const": "warn",
    "prefer-rest-params": "warn",
    "no-useless-escape": "warn",
    "no-irregular-whitespace": "warn",
    "no-prototype-builtins": "warn",
    "no-fallthrough": "warn",
    "no-extra-boolean-cast": "warn",
    "no-case-declarations": "warn",
    "no-async-promise-executor": "warn",
  },
  globals: {
    defineProps: "readonly",
    defineEmits: "readonly",
    defineExpose: "readonly",
    withDefaults: "readonly",
  },
};
