import { createI18n } from "vue-i18n";
import { Vant3Config } from "@/i18n/vant3-config";
import { zh } from "@/i18n/project/zh";
import { en } from "@/i18n/project/en";
import { ja } from "@/i18n/project/ja";

import { $commonService } from "@/services/framework/dependency-injection-service";

const messages = {
  zh: zh,
  en: en,
  ja: ja,
};

export const i18n = createI18n({
  locale: localStorage.getItem("i18nLocale") || "zh", // set locale
  fallbackLocale: "zh", // set fallback locale
  messages, // set locale messages
});

//////////////////////////////////////////////
const vant3Config = new Vant3Config();
export const I18nService = {
  install(app: any) {
    app.config.globalProperties.$i18nService = I18nService;
    $commonService.$dependencyInjectionService.provideFun(
      "$i18nService",
      app.config.globalProperties.$i18nService
    );
    I18nService.init();
  },
  vant3Config: vant3Config,
  use: (i18nLocale: string) => {
    localStorage.setItem("i18nLocale", i18nLocale);
    vant3Config.use(i18nLocale);
  },
  // 获取当前语言
  getI18nLocale: () => {
    // todo 要初始化服务的默认配置
    return localStorage.getItem("i18nLocale");
  },
  init() {
    // todo 拿本地的 i18n 语言设置（没有初始化 用os的）
    // todo 拿app os的本地化
  },
};
