// import 'amfe-flexible'
import { createApp } from "vue";
import App from "./App.vue";
import { provide, inject } from "vue";

// vant import
import vant from "vant";
import "vant/lib/index.css";

// vant plugin
import router from "./router";
// vuex
import { store, key } from "@/store";

// 自定义引入
import VConsole from "vconsole";
import { DependencyInjectionService } from "@/services/framework/dependency-injection-service";
import { AppConfigService } from "@/services/framework/app-config-service";
import { StorageService } from "@/services/framework/storage-service";
import { HttpService } from "@/services/framework/http-service";
import { AuthService } from "@/services/framework/auth-service";
import { JsBridgeService } from "@/services/framework/js-bridge-service";
import { UrlService } from "@/services/framework/url-service";
import { I18nService, i18n } from "@/i18n/i18n-service";
import { AsyncValidatorService } from "@/services/framework/async-validator-service";
import { PlatformService } from "@/services/framework/platform-service";
import { ImgService } from "@/services/framework/img-service";

// 自定义组件封装
import { XLoadingService } from "@/services/framework/loading-service";
import { XToastService } from "@/services/framework/toast-service";
import { XDialogService } from "@/services/framework/dialog-service";
import { EventBusService } from "@/services/framework/event-bus-service";
import { XSquareToastService } from "@/services/framework/square-toast-service";

//引入阿里云字体图标CSS
// import "./../public/assets/iconfont/iconfont.css";
//引入echarts
import * as echarts from "echarts";

const app = createApp(App);
// // 挂在到全局
// window.$app = app;

// 自定义全局服务
app.use(DependencyInjectionService);
app.use(AppConfigService);
window.AppConfig = window.$commonService.$configService.getConfig();
app.use(StorageService);
app.use(EventBusService);

app.use(i18n);
app.use(I18nService);
app.use(JsBridgeService);
app.use(PlatformService);

app.use(ImgService);

// 第三方 url 服务
app.use(UrlService);
app.use(HttpService);
app.use(AuthService);
app.use(AsyncValidatorService);
// 动画服务 暂时没启用
// app.use(PagService);

// 组件服务
// todo 改为懒加载使用 考虑封装服务
app.config.globalProperties.$echarts = echarts; //全局使用
app.use(XLoadingService);
app.use(XToastService);
app.use(XDialogService);
app.use(XSquareToastService);
// vuex 传入 injection key
app.use(store, key);
app.use(router);

// TODO vant 全局加载
app.use(vant);

// 自定义使用

// todo 框架生命周期
app.mount("#app");
