/***
 * 通用验证服务 (同步验证器)
 * 验证规则基于 async-validator
 * 具体可参考 https://www.npmjs.com/package/async-validator
 */
import { provide, inject } from "vue";
import Schema from "async-validator";
import { obj } from "@/types";
import { $commonService } from "@/services/framework/dependency-injection-service";
import { descriptor } from "@/services/framework/validator-libs-service";

// 调用服务时要创建这个类
export class AsyncValidatorServiceClass {
  validator;

  constructor() {
    // 创建同步验证器
    // @ts-ignore
    this.validator = new Schema(descriptor);
  }

  /**
   * 执行校验验证
   * @param obj
   * return 有错误返回 object，没错误返回 void
   */
  async doValidate(obj: obj) {
    return new Promise((resolve) => {
      this.validator.validate(obj, (errors, fields) => {
        resolve(errors);
      });
    });
  }

  // 待验证的数据 必须为扁平化的obj
  // 支持 同步与异步的验证
}

export const AsyncValidatorService = {
  install(app: any) {
    app.config.globalProperties.$validatorService =
      new AsyncValidatorServiceClass();
    $commonService.$dependencyInjectionService.provideFun(
      "$validatorService",
      app.config.globalProperties.$validatorService
    );
  },
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**空值不触发正则验证 todo 暂不启用**/
export function doValidatorIgnoreNull(
  value: any,
  regExp: RegExp
): string | null {
  let isRight = false;
  const errorMsg = "";
  if (!value) {
    return null;
  }

  if (regExp.test(value)) {
    isRight = true;
  }
  return isRight ? null : errorMsg;
}
