import { ResultDto } from "@/dto/result-dto";
import { StorageService } from "@/services/framework/storage-service";
import { $commonService } from "@/services/framework/dependency-injection-service";

/**
 * 鉴权服务
 */
export const AuthService = {
  install(app: any) {
    app.config.globalProperties.$authService = new AuthServiceClass(app);
    $commonService.$dependencyInjectionService.provideFun(
      "$authService",
      app.config.globalProperties.$authService
    );
  },
};

class AuthServiceClass {
  VueApp;
  Http;
  StorageService;
  // JPush;

  // token;
  private tags: any = [];

  constructor(VueApp: any) {
    this.VueApp = VueApp;
    this.Http = this.VueApp.config.globalProperties.$http;
    this.StorageService = this.VueApp.config.globalProperties.$storageService;
    this.StorageService.getData("AuthServiceStore");
  }

  /**
   * 用户登录
   * @param url
   * @param params
   */
  userLogin(url: string, params: any) {
    return new Promise((resolve, reject) => {
      this.Http.get(url, params)
        .then((res: ResultDto<any>) => {
          if (+res.code === 200) {
            resolve(res.data);
          } else {
            reject(res.message);
          }
        })
        .catch((err: any) => {
          console.error("登录失败", JSON.stringify(err));
          reject(err);
        });
    });
  }

  /**
   * 游客登录
   */
  visitorLogin() {}

  /**
   * 注销
   */
  signOut() {}

  /**
   * 退出登录
   */
  loginOut() {
    localStorage.clear();
  }

  /**
   * 获取极光Id
   * todo 迁移 极光推送服务的设备注册id，用于单点登录
   */
  getJgRegId() {
    let jgRegId = this.StorageService.getData("jgRegId");
    if (!jgRegId) {
      jgRegId = Math.random().toString(36).substr(2, 9);
      this.StorageService.setData("jgRegId", jgRegId);
    }
    return jgRegId;
  }
}
