// 原生端注入js的作用域名为“native.nativeApi”;
// js提供给原生回调的作用域名为“nativeApiCallback”;
// js与原生通信列表的作用域名为“nativeApiCallbackList”;
// nativeApi参考命名 文档 https://uniapp.dcloud.net.cn/api/system/info.html
import {
  BusinessBaseClass,
  NativeApiClass,
  statusCode,
} from "@/types/js-bridge";
import { $commonService } from "@/services/framework/dependency-injection-service";
// declare const window: any;

const apiListObj = {
  takePhoto: {}, // 拍照
  photoCompont: {}, // 相册
  scan: {}, // 扫码
  /////////////////////////////////////////////////////////////////
  saveLog: {}, // 读取前端日志并保存在手机本地文件
  closeWebShade: {}, // 安卓引导页webview遮罩页面关闭 （还需要个引导页面路径）
  appVersions: {}, // 获取app版本名称、版本号、唯一id（极光id）
  setting: {}, // 根据不同需求跳转不同的原生设置页面(如bluetooth跳转蓝牙相关)
  location: {}, // 获取当前经纬度（百度坐标，也可以是通用坐标）
  bluetooth: {}, // 打开蓝牙
  /////////////////////////////////////////////////////////////////
  becameState: {}, // 打开前后台切换监听
  setData: {}, // 原生持续化存储设置
  getData: {}, // 原生持续化存储设置
  showToast: {}, // 弹出原生自定义toast
  i18n: {}, // 获取语言配置
  ////////////////////////////////////////////////////////////////
  network: {},
  device: {},
  fontSize: {},
  wxLogin: {},
  saveFile: {},
  openSaveFile: {},
  ////////////////////////////////////////////////////////////////
  scanBle: {},
  connectBle: {},
  writeBle: {},
  readBle: {},
  disConnect: {},
  electricQuantity: {},
  ////////////////////////////////////////////////////////////////
  jpushId: {},
  stopPush: {},
  resumePush: {},
  setJpushAlias: {},
  setJpushTag: {},
  setJpushTags: {},
  receiveJpush: {},

  appleLogin: {}, //苹果登录
  isInstalledWx: {}, //是否安装微信

  getCameraPermissions: {}, //获取相机权限
  getAlbumPermissions: {}, //获取相册权限
  getLocationPermissions: {}, //获取位置权限

  saveBase64ToFile: {}, //base64保存到相册

  stopScan: {}, //停止扫码
  scanToast: {}, //扫码错误提示
  resumeScanning: {}, //恢复扫码
  receiveIsAndroidBack: {}, //监听Android物理返回键
};

export const JsBridgeService = {
  install(app: any) {
    app.config.globalProperties.$jsBridgeService = new JsBridgeServiceClass();
    $commonService.$dependencyInjectionService.provideFun(
      "$jsBridgeService",
      app.config.globalProperties.$jsBridgeService
    );
  },
};

class JsBridgeServiceClass {
  constructor() {
    this.init();
  }

  init() {
    // 初始化js原生通信列表
    this.nativeApiCallbackInit();
    if (!window.native) {
      window.native = {};
    }
    window.native.transformNativeApiOutputData =
      this.transformNativeApiOutputData;
    window.native.nativeApiCallbackPipe = this.nativeApiCallbackPipe;
    window.nativeApiCallback = this.doNativeApiCallback;
    window.nativeApiCallbackList = {};

    // 创建事件订阅器
    // @ts-ignore
    document.addEventListener("nativeApiCallback", this.nativeApiCallbackPipe);
  }

  // js调用NativeApi
  async doNativeApi(
    inputData: NativeApiClass,
    successCallback: Function,
    errorCallback?: Function
  ): Promise<any> {
    console.log("发送数据到原生：", inputData);
    // alert("开始调用");
    const createCallbackFun = this.createCallbackFun(
      inputData,
      successCallback,
      errorCallback
    );

    // 先转json字符串
    const jsonStr = JSON.stringify(inputData);

    // 再转码
    const postData = encodeURIComponent(jsonStr);

    // 调用原生api
    console.warn(window.native);
    console.warn(window.native.nativeApi);

    console.log("发送数据到原生（转码）：", postData);

    if (!window.native.nativeApi && !window.webkit) {
      throw new Error("少年请检查你的运行环境");
    }

    // iso与安卓 调用不同的类
    if (window.webkit && window.webkit.messageHandlers) {
      // window.webkit.messageHandlers.<name>.postMessage(<messageBody>)
      // window.webkit.messageHandlers.nativeApi 这个方法是ios主动监听的
      window.webkit.messageHandlers.nativeApi.postMessage(postData);
    } else {
      window.native.nativeApi(postData);
    }
    return createCallbackFun;
  }

  // 创建回调方法
  createCallbackFun(
    inputData: any,
    successCallback: Function,
    errorCallback?: Function
  ) {
    // 处理methodId
    const methodId = inputData.type + this.getNativeTypeNumber(inputData.type);
    inputData.methodId = methodId;

    console.log("callbackId", methodId);

    window.nativeApiCallbackList[methodId] = {
      successCallback: successCallback,
      errorCallback: errorCallback,
    };
    return window.nativeApiCallbackList[methodId];
  }

  // 获取原生类型id
  getNativeTypeNumber(type: string) {
    if (window.nativeApiCallbackList[type + "Number"]) {
      window.nativeApiCallbackList[type + "Number"] += 1;
      return window.nativeApiCallbackList[type + "Number"];
    }
    window.nativeApiCallbackList[type + "Number"] = 1;
    return 1;
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // 原生执行js回调
  async doNativeApiCallback(inputData: string) {
    // alert("进行了回调");
    console.log("原生回传数据：", inputData);
    // console.log('doNativeApiCallback===== this',this)
    console.log("原生回传数据 window：", window);
    const result: NativeApiClass =
      window.native.transformNativeApiOutputData(inputData);

    const sendEvent = new CustomEvent("nativeApiCallback", { detail: result });
    document.dispatchEvent(sendEvent);
  }

  // NativeApiCallback 管道
  nativeApiCallbackPipe(eventInitDict: any) {
    const result = eventInitDict.detail;
    console.warn("nativeApiCallbackPipe", result);
    const callbackList = window.nativeApiCallbackList[result.methodId];

    // @ts-ignore
    if (result.nativeJsonData.code === "200") {
      callbackList.successCallback(result.nativeJsonData);
    } else {
      console.error(
        "NativeApiOutputData 错误码：",
        // @ts-ignore
        result.nativeJsonData.code,
        // @ts-ignore
        statusCode[result.nativeJsonData.code]
      );

      console.error("NativeApiOutputData 错误", result);
      callbackList.errorCall &&
        callbackList.errorCallback(result.nativeJsonData);
    }

    // todo 内存回收机制 暂时不开启
    // setTimeout(() => {
    //   delete window.nativeApiCallbackList[result.methodId];
    // }, 10 * 1000);
  }

  // 检查NativeApiOutputData,并转码
  transformNativeApiOutputData(nativeApiOutputData: string): NativeApiClass {
    // 先转码
    const json = decodeURIComponent(nativeApiOutputData);
    // 再转换json
    const obj: NativeApiClass = JSON.parse(json);
    console.log("接收到后端数据转换完成：", NativeApiClass);
    return obj;
  }

  // 调用机制-注册机制 根据type创建方法
  nativeApiCallbackInit() {
    Object.keys(apiListObj).forEach((key) => {
      // @ts-ignore
      this[key] = this.createNativeApiCallback(key);
    });
  }

  createNativeApiCallback(type: string): Function {
    const config = new NativeApiClass();
    config.type = type;
    // 接收调用方法传入值
    return (
      jsJsonData: undefined | null | object,
      successCallback: Function,
      errorCallback?: Function
    ) => {
      // 拼装接口调用所需的值
      const nativeApiClass = JSON.parse(JSON.stringify(config));
      nativeApiClass.jsJsonData = jsJsonData ? jsJsonData : {};
      // debugger
      return this.doNativeApi(nativeApiClass, successCallback, errorCallback);
    };
  }
}
