/**
 * 用于原生端与js端通信，方法互相调用的标准接口
 * NativeApiClass 在方法调用时要先转换为json字符串,之后转为Unicode
 * native端开发人员 给出具体业务的type种类与传参约定
 * js端 根据后端给出的type与接口 提供nativeApiCallback作用域与具体可回调的函数
 */
export class NativeApiClass {
  methodId!: string; // 由js生成 规则 methodName + 加自增数字,强制解析 一定有值
  methodName = "nativeApiCallback"; // 原生端回调js的方法名 目前只有一個 nativeApiCallback
  jsJsonData: object = {}; // js传给原生，传参参考 具体type的业务定义; 无数据传{}
  nativeJsonData?: NativeJsonDataInterface<any>; // 原生传给js 传参参考 具体type的业务定义; 无数据传{};
  type!: string; // 原生端业务处理标识
}

// 原生返回给js的数据接口
export interface NativeJsonDataInterface<T> {
  code: string; // 原生执行是否成功  // 200 正常；403 没有权限
  errorMsg: string; // 只有错误时返回
  data: T | BusinessBaseClass<T>; // 业务数据返回多态
}

// 业务基础类
export class BusinessBaseClass<T> {
  type?: "enter" | "cancel"; // 业务动作 确定 enter , 取消 cancel；订阅动作：next （传值）；error （错误处理）；complete （完成/终止）
  data?: T; // 数据
}

// 状态码
export const statusCode = {
  "200": "正常",
  "401": "交互错误",
  "403": "没有权限",
  // 蓝牙部分异常码
  "301": "不支持蓝牙(手机无蓝牙)",
  "302": "蓝牙连接异常",
  "303": "蓝牙写入数据失败",
};
